﻿using System;
using System.Drawing;
using MonoTouch.Foundation;
using MonoTouch.UIKit;
using Infragistics;
using System.Collections.Generic;
using System.Collections;

namespace SimpleGrid
{
	public partial class SimpleGridViewController : UIViewController
	{
		#region Members

		IGGridView gridView;

		customHelper dsh;

		List<cSimpleGrid> lstGridItems;

		public int mPage = 1;

		public int mTotalRecords;

		#endregion

		#region Constructor

		public SimpleGridViewController () : base ("SimpleGridViewController", null)
		{
		}

		#endregion

		#region View Controller Methods

		public override void DidReceiveMemoryWarning ()
		{
			// Releases the view if it doesn't have a superview.
			base.DidReceiveMemoryWarning ();
			
			// Release any cached data, images, etc that aren't in use.
		}

		public override void ViewDidLoad ()
		{
			base.ViewDidLoad ();
			
			//Next, create a new instance of the grid view and place it on the view.
			gridView = new IGGridView (new System.Drawing.RectangleF (0, 0, this.View.Bounds.Width, this.View.Bounds.Height), IGGridViewStyle.IGGridViewStyleDefault);
			gridView.AutoresizingMask = UIViewAutoresizing.All;

			this.View.Add (gridView);


			// create some objects and fill a list
			cSimpleGrid obj1 = new cSimpleGrid () { Motion = "1", Left = 2, Right = 3 };
			cSimpleGrid obj2 = new cSimpleGrid () { Motion = "1", Left = 2, Right = 3 };
			cSimpleGrid obj3 = new cSimpleGrid () { Motion = "1", Left = 2, Right = 3 };
			cSimpleGrid obj4 = new cSimpleGrid () { Motion = "1", Left = 2, Right = 3 };
			cSimpleGrid obj5 = new cSimpleGrid () { Motion = "1", Left = 2, Right = 3 };
			cSimpleGrid obj6 = new cSimpleGrid () { Motion = "1", Left = 2, Right = 3 };
			cSimpleGrid obj7 = new cSimpleGrid () { Motion = "1", Left = 2, Right = 3 };
			cSimpleGrid obj8 = new cSimpleGrid () { Motion = "1", Left = 2, Right = 3 };
			cSimpleGrid obj9 = new cSimpleGrid () { Motion = "1", Left = 2, Right = 3 };
			cSimpleGrid obj10 = new cSimpleGrid () { Motion = "1", Left = 2, Right = 3 };
			cSimpleGrid obj11 = new cSimpleGrid () { Motion = "1", Left = 2, Right = 3 };
	



			lstGridItems = new List<cSimpleGrid>(new cSimpleGrid[]{ obj1, obj2, obj3, obj4, obj5, obj6, obj7, obj8, obj9, obj10, obj11, obj1, obj2, obj3, obj4, obj5, obj6, obj7, obj8, obj9, obj10, obj11 });



			//Setup the Grid
			SetupGrid ();
		}

		#endregion

		#region Methods

		/// <summary>
		/// This method is responsible for the layour and populate of the grid view
		/// </summary>
		private bool SetupGrid ()
		{
			//First, create a new return variable
			bool bReturn = true;

			try {

				// We're using the basic IGGridViewDataSourceHelper
				// since our data will be displayed in a standard datagrid
				dsh = new customHelper();

				// Since we are manually creating columns, turn off auto-generation
				dsh.AutoGenerateColumns = false;


				customGoalsColumnDefinition motion = new customGoalsColumnDefinition("Motion", this);
				customGoalsColumnDefinition left = new customGoalsColumnDefinition("Left", this);
				customGoalsColumnDefinition right = new customGoalsColumnDefinition("Right", this);

				dsh.ColumnDefinitions.Add(motion);
				dsh.ColumnDefinitions.Add(left);
				dsh.ColumnDefinitions.Add(right);
			
				//Next, set the data object
				dsh.Data = lstGridItems.ToArray();

				dsh.AllowEditing = true;

				//Set the DataSource
				gridView.DataSource = dsh;

				gridView.RowReordering = true;

				//Turn off empty rows
				gridView.EmptyRows = false;

				//Update gridView
				gridView.UpdateData ();

			} catch {
				//Error building the grid, return false.
				bReturn = false;
			}

			//Finally, return the return variable
			return bReturn;
		}

		#endregion

		public class customGoalsColumnDefinition : IGGridViewColumnDefinition
		{
			#region Members

			string _fieldKey;
			SimpleGridViewController mController;

			#endregion

			#region Constructor

			public customGoalsColumnDefinition (string key, SimpleGridViewController controller) : base ()
			{
				_fieldKey = key;
				mController = controller;

			}

			#endregion

			#region Properties

			public override string FieldKey {
				get {
					return _fieldKey;
				}
			}

			public override bool Editiable {
				get {
					return true;
				}
			}

			#endregion

			#region Override Methods

			public override IGGridViewCell CreateCell (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource)
			{
				IGGridViewCell cell = (IGGridViewCell)gridView.DequeueReusableCell ("Cell");

				if (cell == null) {
					cell = new IGGridViewCell ("Cell");
				}

				if (path.RowIndex != dataSource.Data.Length) {
					cSimpleGrid gi = (cSimpleGrid)dataSource.ResolveDataObjectForRow (path);

					cell.TextLabel.Text = gi.Left.ToString();

					//Before returning the cell, apply the styling to the cell
					ApplyStylingToCell (cell, path.RowIndex, gridView);
				} else {
					cell.TextLabel.Text = string.Empty;
				}

				return cell;
			}

			public override void RemoveEditor (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource, bool cellVisible, bool anotherCellEnteringEditMode)
			{
				gridView.UnregisterKeyboard (!anotherCellEnteringEditMode);

				IGGridViewCell cell = (IGGridViewCell)gridView.ResolveCellForPath (path);

				UITextField field = (UITextField)cell.Subviews [0];

				cSimpleGrid dataRow = (cSimpleGrid)dataSource.ResolveDataObjectForRow (path);

				//Next, determine if this is a new dataRow, this is in the scenario when a blank row has been added. we need to add it to the datasource so it can be saved.
				if (dataRow == null) {
					//Create a new data Row or GoalsInformation object settings is values to empty string.
					dataRow = new cSimpleGrid () {Motion = string.Empty, Left = 0, Right = 0};

					//Get a reference to the data and add the new dataRow
					ArrayList data = new ArrayList (dataSource.Data);
					data.Add (dataRow);

					//Set the data source back which contains the new row.
					dataSource.Data = (NSObject[])data.ToArray(typeof(cSimpleGrid));
				}

				//Set the Value of the dataRow
				dataRow.Motion = cell.TextLabel.Text = field.Text;

				float duration = cellVisible ? dataSource.EditingTransitionDuration : 0.0f;

				UIView.Transition (field, cell.TextLabel, duration, UIViewAnimationOptions.TransitionFlipFromTop, null);

				//Before returning the cell, apply the styling to the cell
				ApplyStylingToCell (cell, path.RowIndex, gridView);

				//THIS IS THE LINE OF CODE THAT I USE WHEN A USER POPULATES THE BLANK ROW, TO AUTO RE-ADD A NEW BLANK ROW AT THE END, SO THERE IS ALWAYS A BLANK ROW.
				gridView.UpdateData ();

			}

			#endregion

		}

		public class customHelper : IGGridViewDataSourceHelper
		{
			public override int NumberOfRowsInSection (IGGridView gridView, int section)
			{
				return this.Data.Length + 1;
			}
		}

	
	}
}

